# Filename input plugin for Embulk


## Overview

* **Plugin type**: input
* **Resume supported**: no
* **Cleanup supported**: yes
* **Guess supported**: no

## Configuration

- **multi_dir**: description (ArrayList<String>, required)
- **mulit_tag**: description (ArrayList<String>, default: `[]`)
- **load_order**: description (String, default: `ALPHABETICAL`)
- **chunk_size**: description (int, default: `10485760(10M)`)

Attention: For the order option. There are many alternative:
ALPHABETICAL (default value)
ASCEND_MODIFIED
DESCEND_MODIFIED
ASCEND_CREATION (useless in the unix system, because the unix does not record the creation time)
DESCEND_CREATION (useless in the unix system)

## Example

```yaml
exec:
  min_output_tasks: 1
in:
  type: filename 
  multi_dir: ["../sample/sample_","../example/example_"]
  multi_tag: ["tag1","tag2"]
  load_order: ASCEND_MODIFIED
  chunk_size: 1000
```
Attention:

**exec:**
  **min_output_tasks: 1**

This configuration is oblige!
Embulk will optimize the task according the core number of the PC which means that, it will re-distribute the task and cause errors.
If the multi_dir contains more than one directory, each directory will be treated as a task. the Embulk will distribute those tasks to multi 
thread. Each task will run consistently, the files in each directory will be uploading in order.
For example the upload order maybe:
- example1.txt
- sample1.txt
- sample2.txt
- example2.txt
- sample3.txt

If you want to upload the directory one by one, you need to configure the 
**max_thread: 1**

then you will get
- example1.txt
- example2.txt
- sample1.txt
- sample2.txt
- sample3.txt


## Build
java 1.8 is required.

```
$ ./gradlew gem  # -t to watch change of files and rebuild continuously
```

## Usage

If you are a new user of embulk.
Here are some tips can help you use this plugin quickly.

First of all, you need to have a java8 environment in a linux system.

And you need a erp5 isntance. [if not, follow the tutorial to have 
one](https://nexedi.erp5.net/web_page_module/7056/WebPage_view?ignore_layout:int=1&selection_index=0&portal_status_message=Status%20changed.&selection_name=web_page_module_view_web_page_list_selection&editable_mode:int=1)

Then, you need a embulk on your PC, now there is a bug to load the plugin with the newest embulk. I recommand you use the embulk_v.8.27 instead 
of the newest version.

To install the embulk

```
curl --create-dirs -o ~/.embulk/bin/embulk -L "https://dl.bintray.com/embulk/maven/embulk-0.8.27.jar"
chmod +x ~/.embulk/bin/embulk
echo 'export PATH="$HOME/.embulk/bin:$PATH"' >> ~/.bashrc
source ~/.bashrc
```



After installing the embulk. You need to build this filename-input-plugin on your PC.
```
git clone https://lab.nexedi.com/caiyu/embulk-input-filename/tree/multiThread
cd embulk-input-filename
./gradlew package
```

If you want to test the Plugin. Just run ./gradlew test
In fact this input should be used with the wendelin-output-plugin. build it on your PC too.
```
git https://lab.nexedi.com/caiyu/embulk-output-wendelin/tree/java-output
cd embulk-output-plugin
./gradlew package
```

Now you can use the embulk with these two plugin to upload the data.

In your workplace, create a yml file. Say that we create a config.yml, and fill in the configuration.
```yaml
exec:
  min_output_tasks: 1
in:
  type: filename
  mulit_dir: ["../sample/sample_","../example/example_"]
  multi_tag: ["tag1","tag2"]
  load_order: ASCEND_MODIFIED
  chunk_size: 1000
out:
  type: wendelin
  tag: "weather-cc"
  streamtool_uri: https://softinstxxxxx.host.vifib.net/erp5/portal_ingestion_policies/weather-cc 
  user: zope
  password: yourpassword
```
Prepare the sample data and example data to upload. 
```
mkdir ../sample
vim ../sample/sample_01.txt
vim ../sample/sample_02.txt
vim ../sample/sample_03.txt
mkdir ../example
vim ../example/example_01.txt
vim ../example/example_02.txt
```
Then run the embulk
```
embulk run -L path/to/embulk-input-filename -L path/to/embulk-output-wendelin config.yml
```

 







